/* eslint-disable jsx-a11y/alt-text */
import React, { useState, useEffect } from "react";
import logoDefault from "./logo.svg";
import "./App.scss";
import { Provider, connect } from "react-redux";
import { createStore, combineReducers, applyMiddleware } from "redux";
import thunk from "redux-thunk";
import "../node_modules/bootstrap/dist/css/bootstrap.min.css";
import { Button } from "react-bootstrap";
import { Router, Route, Link, Switch, withRouter } from "react-router-dom";
import { createBrowserHistory } from "history";
import {
  Navbar,
  NavDropdown,
  Container,
  Nav,
  Form,
  Row,
  Col,
} from "react-bootstrap";

const jwtDecode = (token) => {
  try {
    let arrToken = token.split(".");
    let base64Token = atob(arrToken[1]);
    return JSON.parse(base64Token);
  } catch (e) {
    console.log("Error JWT: " + e);
  }
};

const getGQL =
  (url) =>
  async (query, variables = {}) => {
    let obj = await fetch(url, {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
        Authorization: localStorage.authToken
          ? "Bearer " + localStorage.authToken
          : {},
      },
      body: JSON.stringify({ query, variables }),
    });
    let a = await obj.json();
    if (!a.data && a.errors) throw new Error(JSON.stringify(a.errors));
    return a.data[Object.keys(a.data)[0]];
  };

const backURL = "http://shop-roles.asmer.fs.a-level.com.ua";

const gql = getGQL(backURL + "/graphql");

const actionRootCats = () =>
  actionPromise(
    "rootCats",
    gql(`query {
            CategoryFind(query: "[{\\"parent\\":null}]"){
                _id name
            }
        }`)
  );

const actionCatById = (_id) =>
  actionPromise(
    "catById",
    gql(
      `query catById($q: String){
      CategoryFindOne(query: $q){
          _id name,
          goods{
            _id name price images {
               url
            }
          },
          subCategories{
            name, subCategories{
              name
            }
          }
        }
  }`,
      { q: JSON.stringify([{ _id }]) }
    )
  );

const actionGoodById = (_id) =>
  actionPromise(
    "goodById",
    gql(
      `query goodById($q: String){
  GoodFindOne(query: $q){
          _id name description price images{
            url
          }
      }
  }`,
      { q: JSON.stringify([{ _id }]) }
    )
  );

const actionPromise = (name, promise) => async (dispatch) => {
  dispatch(actionPending(name));
  try {
    let data = await promise;
    dispatch(actionResolved(name, data));
    return data;
  } catch (error) {
    dispatch(actionRejected(name, error));
  }
};

const actionPending = (name) => ({ type: "PROMISE", status: "PENDING", name });
const actionResolved = (name, payload) => ({
  type: "PROMISE",
  status: "RESOLVED",
  name,
  payload,
});
const actionRejected = (name, error) => ({
  type: "PROMISE",
  status: "REJECTED",
  name,
  error,
});

const actionLogin = (login, password) =>
  actionPromise(
    "login",
    gql(
      `query NameForMe1($login:String, $password:String){
            login(login:$login, password:$password)
        }`,
      { login, password }
    )
  );

const actionRegister = (login, password) =>
  actionPromise(
    "registration",
    gql(
      `mutation reg2($user:UserInput) {
      UserUpsert(user:$user) {
      _id login
      }
  }
  `,
      { user: { login: login, password: password } }
    )
  );

const actionFullLogin = (login, password) => async (dispatch) => {
  let token = await dispatch(actionLogin(login, password));
  if (token) {
    dispatch(actionAuthLogin(token));
  }
};

const actionFullRegister = (login, password) => async (dispatch) => {
  let check = await dispatch(actionRegister(login, password));
  if (check) {
    dispatch(actionFullLogin(login, password));
  }
};

const actionAuthLogin = (token) => ({ type: "AUTH_LOGIN", token });
const actionAuthLogout = () => ({ type: "AUTH_LOGOUT" });

function cartReducer(state = {}, { type, good = {}, count = 1 }) {
  const { _id } = good;
  const types = {
    CART_ADD() {
      count = +count;
      if (!count) return state;
      return {
        ...state,
        [_id]: {
          good,
          count: count + (state[_id]?.count || 0),
        },
      };
    },
    CART_CHANGE() {
      count = +count;
      if (!count) return state;
      return {
        ...state,
        [_id]: {
          good,
          count: count,
        },
      };
    },
    CART_REMOVE() {
      let { [_id]: remove, ...newState } = state;
      return {
        ...newState,
      };
    },
    CART_CLEAR() {
      return {};
    },
  };
  if (type in types) {
    return types[type]();
  }
  return state;
}

const actionCartAdd = (good, count = 1) => ({
  type: "CART_ADD",
  good,
  count,
});

const actionCartChange = (good, count) => ({
  type: "CART_CHANGE",
  good,
  count,
});

const actionCartRemove = (good) => ({ type: "CART_REMOVE", good });

const actionCartClear = () => ({ type: "CART_CLEAR" });

function promiseReducer(state = {}, { type, status, payload, error, name }) {
  if (type === "PROMISE") {
    return {
      ...state,
      [name]: { status, payload, error },
    };
  }
  return state;
}

function authReducer(state, { type, token }) {
  if (!state) {
    if (localStorage.authToken) {
      type = "AUTH_LOGIN";
      token = localStorage.authToken;
    } else state = {};
  }
  if (type === "AUTH_LOGIN") {
    localStorage.setItem("authToken", token);
    let payload = jwtDecode(token);
    if (typeof payload === "object") {
      return {
        ...state,
        token,
        payload,
      };
    } else return state;
  }
  if (type === "AUTH_LOGOUT") {
    localStorage.removeItem("authToken");
    return {};
  }
  return state;
}

const store = createStore(
  combineReducers({
    promise: promiseReducer,
    auth: authReducer,
    cart: cartReducer,
  }),
  applyMiddleware(thunk)
);

store.subscribe(() => console.log(store.getState()));
store.dispatch(actionRootCats());

const Logo = ({ logo = logoDefault }) => (
  <Link to="/" className="navbar-brand">
    <img src={logo} width="30" height="30" alt="" />
    React Site
  </Link>
);

const Cart = ({ cart }) => {
  let count = 0;
  let sum = Object.entries(cart).map(([, val]) => val.count);
  count = sum.reduce((a, b) => a + b, 0);
  return (
    <Link className="nav-link" to="/cart">
      Корзина: {count} товара
    </Link>
  );
};

const CCart = connect(({ cart }) => ({ cart }))(Cart);

const RootCategory = ({ cat: { _id, name } = {} }) => (
  <NavDropdown.Item
    componentclass={Link}
    href={`/category/${_id}`}
    to={`/category/${_id}`}
  >
    {name}
  </NavDropdown.Item>
);

const RootCategories = ({ cats }) => (
  <NavDropdown title="Категории" id="collasible-nav-dropdown">
    {cats.map((cat, i) => (
      <RootCategory cat={cat} key={i} />
    ))}
  </NavDropdown>
);

const CRootCategories = connect((state) => ({
  cats: state.promise.rootCats?.payload || [],
}))(RootCategories);

const Header = () => (
  <Navbar collapseOnSelect expand="lg" bg="dark" variant="dark">
    <Container>
      <Navbar.Brand>
        <Logo />
      </Navbar.Brand>
      <Navbar.Toggle aria-controls="responsive-navbar-nav" />
      <Navbar.Collapse id="responsive-navbar-nav">
        <Nav className="me-auto">
          <CRootCategories />
        </Nav>
        <Nav>
          <CCart />
          <СAuthorizations />
        </Nav>
      </Navbar.Collapse>
    </Container>
  </Navbar>
);

const Footer = ({ logo = logoDefault }) => (
  <footer>
    <Logo logo={logo} />
  </footer>
);

const Content = ({ children }) => <div className="Content">{children}</div>;

const SubCategories = ({ cats } = {}) => {
  console.log(...cats);
  return (
    <div className="SubCategories">
      <h3>Подкатегории</h3>
    </div>
  );
};

const GoodCard = ({ good: { _id, name, price, images } = {}, onCartAdd }) => (
  <div className="GoodCard">
    <h2>{name}</h2>
    {images && images[0] && images[0].url && (
      <img src={backURL + "/" + images[0].url} />
    )}
    <p>Цена: {price}</p>
    <Link to={`/good/${_id}`}>Посмотреть</Link>
    <br />
    <Button
      variant="success"
      onClick={() => onCartAdd({ _id, name, price, images })}
    >
      +
    </Button>
  </div>
);

const CGoodCard = connect(null, { onCartAdd: actionCartAdd })(GoodCard);

const Category = ({ cat: { _id, name, goods, subCategories } }) => (
  <div className="Category">
    <h1>{name}</h1>
    {subCategories && <SubCategories cats={subCategories} />}
    {(goods || []).map((good, i) => (
      <CGoodCard good={good} key={i} />
    ))}
  </div>
);

const CCategory = connect((state) => ({
  cat: state.promise.catById?.payload || [],
}))(Category);

const CartGood = ({
  cartGood: { good, count },
  removeCartGood,
  changeCartGood,
}) => (
  <div className="CartGood">
    <div className="GoodImgBlock">
      {good.images && good.images[0].url && (
        <img src={backURL + "/" + good.images[0].url} />
      )}
    </div>
    <div className="GoodInfoBlock">
      <h2>{good.name}</h2>
      <p>Цена: {good.price * count}$</p>
      <p>Всего: {count} шт.</p>
    </div>
    <div className="GoodEditBlock">
      <Button
        variant="warning"
        id="DeleteBtn"
        onClick={() => removeCartGood(good)}
      >
        Delete
      </Button>
      <label hrmlFor="CountField">Change count</label>
      <input
        type="number"
        value={count}
        min="1"
        id="CountField"
        onInput={(e) => changeCartGood(good, e.currentTarget.value)}
      />
    </div>
  </div>
);

const CCartGood = connect(null, {
  removeCartGood: actionCartRemove,
  changeCartGood: actionCartChange,
})(CartGood);

const CartPage = ({ cart, clearCart }) => (
  <div className="CartMain">
    <h1>Cart</h1>

    {Object.entries(cart).map(([, cartGood]) => (
      <CCartGood cartGood={cartGood} />
    ))}

    {Object.keys(cart).length === 0 ? (
      <h2>Your cart is empty!</h2>
    ) : (
      <Button variant="danger" id="ClearBtn" onClick={() => clearCart()}>
        Clear Cart
      </Button>
    )}
  </div>
);

const CCartPage = connect((state) => ({ cart: state.cart }), {
  clearCart: actionCartClear,
})(CartPage);

const LoginForm = ({ onLogin, onRegister }) => {
  const [login, setLogin] = useState("");
  const [password, setPassword] = useState("");
  return (
    <div className="LoginForm mx-auto mt-5">
      <Form>
        <Form.Group as={Row} className="mb-3" controlId="formHorizontalEmail">
          <Form.Label column sm={2}>
            Login
          </Form.Label>
          <Col sm={10}>
            <Form.Control
              type="text"
              placeholder="Your login"
              style={{ borderColor: login.length < 1 ? "red" : "" }}
              onChange={(e) => setLogin(e.target.value)}
            />
          </Col>
        </Form.Group>
        <Form.Group as={Row} className="mb-3" controlId="formHorizontalEmail">
          <Form.Label column sm={2}>
            Password
          </Form.Label>
          <Col sm={10}>
            <Form.Control
              type="password"
              placeholder="Your password"
              style={{ borderColor: password.length < 1 ? "red" : "" }}
              onChange={(e) => setPassword(e.target.value)}
            />
          </Col>
        </Form.Group>
        <Form.Group as={Row} className="mb-3">
          <Col sm={{ span: 10, offset: 2 }}>
            <Button
              variant="success"
              disabled={login.length < 1 || password.length < 1 ? true : false}
              onClick={() => onLogin(login, password)}
            >
              Login
            </Button>{" "}
            <Button
              variant="secondary"
              disabled={login.length < 1 || password.length < 1 ? true : false}
              onClick={() => onRegister(login, password)}
            >
              Register
            </Button>
          </Col>
        </Form.Group>
      </Form>
    </div>
  );
};

const CLoginForm = connect(null, {
  onLogin: actionFullLogin,
  onRegister: actionFullRegister,
})(LoginForm);

const PageMain = () => <h1>Главная страница</h1>;

const PageCategory = ({
  match: {
    params: { _id },
  },
  getData,
  history,
}) => {
  useEffect(() => {
    getData(_id);
  }, [_id]);
  return <CCategory />;
};
const CPageCategory = connect(null, { getData: actionCatById })(PageCategory);

const Good = ({
  good: { _id, name, description, price, images } = {},
  onCartAdd,
}) => {
  return (
    <div className="good">
      <h1>Страница товара</h1>
      <h1>{name}</h1>
      {images && images[0] && images[0].url && (
        <img src={backURL + "/" + images[0].url} />
      )}
      <p>
        <strong>Описание:</strong> {description}
      </p>
      <strong>Цена: {price} грн.</strong>
      <br />
      <Button
        variant="success"
        onClick={() => onCartAdd({ _id, name, price, images })}
      >
        Добавить в корзину
      </Button>
    </div>
  );
};
const CGood = connect((state) => ({ good: state.promise.goodById?.payload }), {
  onCartAdd: actionCartAdd,
})(Good);

const PageGood = ({
  match: {
    params: { _id },
  },
  getData,
}) => {
  useEffect(() => {
    getData(_id);
  }, [_id]);
  return <CGood />;
};
const CPageGood = connect(null, { getData: actionGoodById })(PageGood);

const Authorizations = ({ auth, actionLogOut }) => {
  if (auth.payload) {
    return (
      <div>
        <strong>Добро пожаловать, {auth.payload.sub.login}</strong>
        <Button variant="danger" onClick={() => actionLogOut()}>
          Выйти
        </Button>
      </div>
    );
  } else {
    return (
      <Link className="nav-link" to={`/authorizations`}>
        Авторизоваться
      </Link>
    );
  }
};

const СAuthorizations = connect((state) => ({ auth: state.auth }), {
  actionLogOut: actionAuthLogout,
})(Authorizations);

const Page404 = () => (
  <>
    <h1>404 Error Page</h1>
    <p className="zoom-area">Page was not found</p>
    <section className="error-container">
      <span className="four">
        <span className="screen-reader-text">4</span>
      </span>
      <span className="zero">
        <span className="screen-reader-text">0</span>
      </span>
      <span className="four">
        <span className="screen-reader-text">4</span>
      </span>
    </section>
    <div className="link-container">
      <Link to="/" className="more-link">
        Go to home
      </Link>
    </div>
  </>
);

const Main = () => (
  <main>
    <Content>
      <Switch>
        <Route path="/" component={withRouter(PageMain)} exact />
        <Route path="/category/:_id" component={withRouter(CPageCategory)} />
        <Route path="/cart" component={withRouter(CCartPage)} />
        <Route path="/good/:_id" component={withRouter(CPageGood)} />
        <Route path="/authorizations" component={withRouter(CLoginForm)} />
        <Route path="" component={withRouter(Page404)} />
      </Switch>
    </Content>
  </main>
);

const history = createBrowserHistory();

function App() {
  return (
    <Router history={history}>
      <Provider store={store}>
        <div className="App">
          <Header />
          <Main />
          <Footer />
        </div>
      </Provider>
    </Router>
  );
}

export default App;
