import React, { useState, useEffect } from "react";
import "./App.scss";
import watch from "./images/ClockFace.png";
import secondArrow from "./images/ClockFace_S.png";
import minuteArrow from "./images/ClockFace_M.png";
import hourArrow from "./images/ClockFace_H.png";
import logoDefault from "./logo.svg";
import { Button } from "react-bootstrap";

// RGBInput
const RGBInput = () => {
  const [red, setRed] = useState(0);
  const [green, setGreen] = useState(0);
  const [blue, setBlue] = useState(0);
  const color = `rgba(${red},${green},${blue},1)`;

  useEffect(() => {
    console.log("component did mount");
    return () => {
      console.log("component will unmount");
    };
  }, [red]);

  const bounds = (x) => (x < 0 ? 0 : x > 255 ? 255 : x);
  return (
    <div style={{ backgroundColor: color }}>
      <input
        type="number"
        min="0"
        max="255"
        value={red}
        onChange={(e) => setRed(bounds(+e.target.value))}
      />
      <input
        type="number"
        min="0"
        max="255"
        value={green}
        onChange={(e) => setGreen(bounds(+e.target.value))}
      />
      <input
        type="number"
        min="0"
        max="255"
        value={blue}
        onChange={(e) => setBlue(bounds(+e.target.value))}
      />
    </div>
  );
};

// Spoiler
const Spoiler = ({ header = <h4>+</h4>, open, children }) => {
  const [visible, setVisible] = useState(open);
  return (
    <div className="Spoiler">
      <div className="header" onClick={(e) => setVisible(!visible)}>
        <h3>
          {header}
          {visible ? "hide" : "show"}
        </h3>
        {visible && children}
      </div>
    </div>
  );
};

// RangeInput

const RangeInput = ({ min = 1, max = 10 }) => {
  const [len, setLen] = useState(min);
  return (
    <div className="RangeInput">
      <label htmlFor="myInput">
        {len < min ? `The length should be min: ${min}` : ""}
        {len > max ? `The length should be max: ${max}` : ""}
      </label>
      <input
        style={{ borderColor: len < min || len > max ? "red" : "" }}
        type="text"
        onInput={(e) => setLen(e.target.value.length)}
        placeholder="Type your text"
        id="myInput"
      />
    </div>
  );
};

// PasswordConfirm

const PasswordConfirm = ({ min = 4 }) => {
  const [len, setLen] = useState(min);
  const [firstPass, setFirstPass] = useState("");
  const [secondPass, setSecondPass] = useState("");
  return (
    <div className="PasswordConfirm">
      <label htmlFor="passFirst">
        {len < min
          ? `The length should be min: ${min}`
          : !/[A-Z]/g.test(firstPass)
          ? "The password must contain at least one capital letter"
          : "Enter your password"}
      </label>
      <input
        style={{ borderColor: len < min ? "red" : "" }}
        type="password"
        onInput={(e) => setLen(e.target.value.length)}
        onChange={(e) => setFirstPass(e.target.value)}
        id="passFirst"
      />
      <label htmlFor="passSecond">
        {firstPass !== secondPass
          ? "Passwords must match"
          : "Repeat your password"}
      </label>
      <input
        style={{
          borderColor: len < min || firstPass !== secondPass ? "red" : "",
        }}
        type="password"
        onInput={(e) => setLen(e.target.value.length)}
        onChange={(e) => setSecondPass(e.target.value)}
        id="passSecond"
      />
    </div>
  );
};

// Timers

const Timer = ({
  ms = 1000,
  onDelete = null,
  h = null,
  m = null,
  s = null,
}) => {
  const [counter, setCounter] = useState(0);
  const [pause, setPause] = useState(false);

  // counter <= 0 ? setPause(true) : null;
  useEffect(() => {
    const interval = setInterval(() => {
      if (!pause) {
        setCounter((counter) => counter + 1);
      }
    }, ms);
    return () => clearInterval(interval);
  }, [pause, ms]);

  let seconds = s ? s + counter / 1000 : counter / 1000;
  let minutes = m ? m + seconds / 60 : seconds / 60;
  let hours = h ? h + minutes / 60 : minutes / 60;

  return (
    <>
      <h3>
        Hours: {hours.toFixed(3)} | Minutes: {minutes.toFixed(3)} | Seconds:{" "}
        {seconds.toFixed(3)}
      </h3>
      {onDelete && <button onClick={onDelete}>x</button>}
      <button
        onClick={() => {
          setPause(!pause);
        }}
      >
        {pause ? "Continue" : "Pause"}
      </button>
    </>
  );
};

const Timers = () => {
  const [timers, setTimers] = useState([]);
  const [ms, setMS] = useState(1000);
  return (
    <>
      <button onClick={() => setMS(ms + 100)}>+</button>
      {ms}
      <button onClick={() => setMS(ms - 100)}>-</button>
      <button onClick={() => setTimers([Math.random(), ...timers])}>ADD</button>
      {timers.map((i) => (
        <Timer
          key={i}
          ms={ms}
          onDelete={() => setTimers(timers.filter((t) => t !== i))}
        />
      ))}
    </>
  );
};

const TimerControl = () => {
  const [seconds, setSeconds] = useState(0);
  const [minutes, setMinutes] = useState(0);
  const [hours, setHours] = useState(0);
  const [start, setStart] = useState(false);

  return (
    <div className="TimerControl">
      <label htmlFor="Hours">{hours} hours</label>
      <input
        id="Hours"
        min="0"
        type="number"
        onChange={(e) =>
          setHours(+e.currentTarget.value ? +e.currentTarget.value : 0)
        }
      />{" "}
      <label htmlFor="Minutes">{minutes} minutes</label>
      <input
        id="Minutes"
        min="0"
        type="number"
        onChange={(e) =>
          setMinutes(+e.currentTarget.value ? +e.currentTarget.value : 0)
        }
      />{" "}
      <label htmlFor="Seconds">{seconds} seconds</label>
      <input
        id="Seconds"
        min="0"
        type="number"
        onChange={(e) =>
          setSeconds(+e.currentTarget.value ? +e.currentTarget.value : 0)
        }
      />{" "}
      <button onClick={() => setStart(!start)}>
        {start ? "Hide" : "Start"}
      </button>
      {start && <Timer h={hours} m={minutes} s={seconds} />}
    </div>
  );
};

const TimerContainer = ({
  seconds,
  refresh,
  render: Render,
  mode = "minus",
}) => {
  const firstTimer = performance.now();
  const [time, setTime] = useState(seconds);
  useEffect(() => {
    const interval = setInterval(() => {
      let secondTimer = performance.now();
      let timerSeconds = (secondTimer - firstTimer) / 1000;
      if (mode === "minus") {
        if (seconds >= timerSeconds) {
          setTime(seconds - timerSeconds);
        } else {
          setTime(0);
          clearInterval(interval);
        }
      } else {
        setTime(seconds + timerSeconds);
      }
    }, refresh);
    return () => {
      setTime(seconds);
      clearInterval(interval);
    };
  }, [seconds]);
  return <Render seconds={time.toFixed(1)} />;
};
const SecondsTimer = ({ seconds }) => <h2>{seconds}</h2>;

const TimerLCD = ({ seconds = 3600 }) => {
  let sec = seconds * 1;
  let minutes = seconds / 60;
  let hours = minutes / 60;
  return (
    <h4>
      Hours: {hours.toFixed(1)} | Minutes: {minutes.toFixed(1)} | Seconds:{" "}
      {sec.toFixed(1)}
    </h4>
  );
};

const Watch = ({ seconds }) => {
  let ms = seconds * 1000;
  let secondAngle = ms * 0.006;
  let minuteAngle = ms * 0.0001;
  let hourAngle = ms * 0.00000166666;

  return (
    <div className="Watch">
      <img src={watch} alt="Watch" />
      <img
        className="Arrows"
        style={{ transform: `rotate(${secondAngle}deg)` }}
        src={secondArrow}
        alt="Seconds"
      />
      <img
        className="Arrows"
        style={{ transform: `rotate(${minuteAngle}deg)` }}
        src={minuteArrow}
        alt="Minutes"
      />
      <img
        className="Arrows"
        style={{ transform: `rotate(${hourAngle}deg)` }}
        src={hourArrow}
        alt="Hours"
      />
    </div>
  );
};

const TimerControlContainer = ({ refresh, render: Render }) => {
  const firstTimer = performance.now();
  const [seconds, setSeconds] = useState(0);
  const [start, setStart] = useState(false);
  const [time, setTime] = useState(seconds);

  useEffect(() => {
    const interval = setInterval(() => {
      let secondTimer = performance.now();
      let timerSeconds = (secondTimer - firstTimer) / 1000;
      if (seconds >= timerSeconds) {
        setTime(seconds - timerSeconds);
      } else {
        setTime(0);
        clearInterval(interval);
      }
    }, refresh);
    return () => {
      setTime(seconds);
      clearInterval(interval);
    };
  }, [seconds]);

  return (
    <div className="TimerControl">
      <label for="Seconds">{seconds} seconds</label>
      <input
        id="Seconds"
        min="0"
        type="number"
        onChange={(e) =>
          setSeconds(+e.currentTarget.value ? +e.currentTarget.value : 0)
        }
      />{" "}
      <button onClick={() => setStart(!start)}>
        {start ? "Hide" : "Start"}
      </button>
      {start && <Render seconds={time.toFixed(1)} />}
    </div>
  );
};

// MySelect
const countries = {
  AF: "Afghanistan",
  AX: "Aland Islands",
  AL: "Albania",
  DZ: "Algeria",
  AS: "American Samoa",
  AD: "Andorra",
  AO: "Angola",
  AI: "Anguilla",
  AQ: "Antarctica",
  AG: "Antigua And Barbuda",
  AR: "Argentina",
  AM: "Armenia",
  AW: "Aruba",
  AU: "Australia",
  AT: "Austria",
  AZ: "Azerbaijan",
  BS: "Bahamas",
  BH: "Bahrain",
  BD: "Bangladesh",
  BB: "Barbados",
  BY: "Belarus",
  BE: "Belgium",
  BZ: "Belize",
  BJ: "Benin",
  BM: "Bermuda",
  BT: "Bhutan",
  BO: "Bolivia",
  BA: "Bosnia And Herzegovina",
  BW: "Botswana",
  BV: "Bouvet Island",
  BR: "Brazil",
  IO: "British Indian Ocean Territory",
  BN: "Brunei Darussalam",
  BG: "Bulgaria",
  BF: "Burkina Faso",
  BI: "Burundi",
  KH: "Cambodia",
  CM: "Cameroon",
  CA: "Canada",
  CV: "Cape Verde",
  KY: "Cayman Islands",
  CF: "Central African Republic",
  TD: "Chad",
  CL: "Chile",
  CN: "China",
  CX: "Christmas Island",
  CC: "Cocos (Keeling) Islands",
  CO: "Colombia",
  KM: "Comoros",
  CG: "Congo",
  CD: "Congo, Democratic Republic",
  CK: "Cook Islands",
  CR: "Costa Rica",
  CI: 'Cote D"Ivoire',
  HR: "Croatia",
  CU: "Cuba",
  CY: "Cyprus",
  CZ: "Czech Republic",
  DK: "Denmark",
  DJ: "Djibouti",
  DM: "Dominica",
  DO: "Dominican Republic",
  EC: "Ecuador",
  EG: "Egypt",
  SV: "El Salvador",
  GQ: "Equatorial Guinea",
  ER: "Eritrea",
  EE: "Estonia",
  ET: "Ethiopia",
  FK: "Falkland Islands (Malvinas)",
  FO: "Faroe Islands",
  FJ: "Fiji",
  FI: "Finland",
  FR: "France",
  GF: "French Guiana",
  PF: "French Polynesia",
  TF: "French Southern Territories",
  GA: "Gabon",
  GM: "Gambia",
  GE: "Georgia",
  DE: "Germany",
  GH: "Ghana",
  GI: "Gibraltar",
  GR: "Greece",
  GL: "Greenland",
  GD: "Grenada",
  GP: "Guadeloupe",
  GU: "Guam",
  GT: "Guatemala",
  GG: "Guernsey",
  GN: "Guinea",
  GW: "Guinea-Bissau",
  GY: "Guyana",
  HT: "Haiti",
  HM: "Heard Island & Mcdonald Islands",
  VA: "Holy See (Vatican City State)",
  HN: "Honduras",
  HK: "Hong Kong",
  HU: "Hungary",
  IS: "Iceland",
  IN: "India",
  ID: "Indonesia",
  IR: "Iran, Islamic Republic Of",
  IQ: "Iraq",
  IE: "Ireland",
  IM: "Isle Of Man",
  IL: "Israel",
  IT: "Italy",
  JM: "Jamaica",
  JP: "Japan",
  JE: "Jersey",
  JO: "Jordan",
  KZ: "Kazakhstan",
  KE: "Kenya",
  KI: "Kiribati",
  KR: "Korea",
  KP: "North Korea",
  KW: "Kuwait",
  KG: "Kyrgyzstan",
  LA: 'Lao People"s Democratic Republic',
  LV: "Latvia",
  LB: "Lebanon",
  LS: "Lesotho",
  LR: "Liberia",
  LY: "Libyan Arab Jamahiriya",
  LI: "Liechtenstein",
  LT: "Lithuania",
  LU: "Luxembourg",
  MO: "Macao",
  MK: "Macedonia",
  MG: "Madagascar",
  MW: "Malawi",
  MY: "Malaysia",
  MV: "Maldives",
  ML: "Mali",
  MT: "Malta",
  MH: "Marshall Islands",
  MQ: "Martinique",
  MR: "Mauritania",
  MU: "Mauritius",
  YT: "Mayotte",
  MX: "Mexico",
  FM: "Micronesia, Federated States Of",
  MD: "Moldova",
  MC: "Monaco",
  MN: "Mongolia",
  ME: "Montenegro",
  MS: "Montserrat",
  MA: "Morocco",
  MZ: "Mozambique",
  MM: "Myanmar",
  NA: "Namibia",
  NR: "Nauru",
  NP: "Nepal",
  NL: "Netherlands",
  AN: "Netherlands Antilles",
  NC: "New Caledonia",
  NZ: "New Zealand",
  NI: "Nicaragua",
  NE: "Niger",
  NG: "Nigeria",
  NU: "Niue",
  NF: "Norfolk Island",
  MP: "Northern Mariana Islands",
  NO: "Norway",
  OM: "Oman",
  PK: "Pakistan",
  PW: "Palau",
  PS: "Palestinian Territory, Occupied",
  PA: "Panama",
  PG: "Papua New Guinea",
  PY: "Paraguay",
  PE: "Peru",
  PH: "Philippines",
  PN: "Pitcairn",
  PL: "Poland",
  PT: "Portugal",
  PR: "Puerto Rico",
  QA: "Qatar",
  RE: "Reunion",
  RO: "Romania",
  RU: "Russian Federation",
  RW: "Rwanda",
  BL: "Saint Barthelemy",
  SH: "Saint Helena",
  KN: "Saint Kitts And Nevis",
  LC: "Saint Lucia",
  MF: "Saint Martin",
  PM: "Saint Pierre And Miquelon",
  VC: "Saint Vincent And Grenadines",
  WS: "Samoa",
  SM: "San Marino",
  ST: "Sao Tome And Principe",
  SA: "Saudi Arabia",
  SN: "Senegal",
  RS: "Serbia",
  SC: "Seychelles",
  SL: "Sierra Leone",
  SG: "Singapore",
  SK: "Slovakia",
  SI: "Slovenia",
  SB: "Solomon Islands",
  SO: "Somalia",
  ZA: "South Africa",
  GS: "South Georgia And Sandwich Isl.",
  ES: "Spain",
  LK: "Sri Lanka",
  SD: "Sudan",
  SR: "Suriname",
  SJ: "Svalbard And Jan Mayen",
  SZ: "Swaziland",
  SE: "Sweden",
  CH: "Switzerland",
  SY: "Syrian Arab Republic",
  TW: "Taiwan",
  TJ: "Tajikistan",
  TZ: "Tanzania",
  TH: "Thailand",
  TL: "Timor-Leste",
  TG: "Togo",
  TK: "Tokelau",
  TO: "Tonga",
  TT: "Trinidad And Tobago",
  TN: "Tunisia",
  TR: "Turkey",
  TM: "Turkmenistan",
  TC: "Turks And Caicos Islands",
  TV: "Tuvalu",
  UG: "Uganda",
  UA: "Ukraine",
  AE: "United Arab Emirates",
  GB: "United Kingdom",
  US: "United States",
  UM: "United States Outlying Islands",
  UY: "Uruguay",
  UZ: "Uzbekistan",
  VU: "Vanuatu",
  VE: "Venezuela",
  VN: "Vietnam",
  VG: "Virgin Islands, British",
  VI: "Virgin Islands, U.S.",
  WF: "Wallis And Futuna",
  EH: "Western Sahara",
  YE: "Yemen",
  ZM: "Zambia",
  ZW: "Zimbabwe",
};

const MySelect = ({ options = countries, value = "YE", onChange }) => (
  <select value={value} onChange={(value) => onChange(value.target.value)}>
    {Object.entries(options).map((option) => (
      <option value={option[0]}>{option[1]}</option>
    ))}
  </select>
);

// Gallery
const Dots = ({ active, count, onClick }) => {
  const dots = [];
  for (let i = 0; i < count; i++) {
    dots.push(
      <div
        className="Dots"
        style={
          i === active
            ? { border: "2px solid red" }
            : { border: "2px solid black" }
        }
        onClick={() => onClick(i)}
      >
        {i + 1}
      </div>
    );
  }
  return <div className="Dots">{dots}</div>;
};

const Gallery = ({
  images = [
    logoDefault,
    "https://go64.ru/upload/quickly/cat-2143332_1280.jpg",
    "https://upload.wikimedia.org/wikipedia/commons/thumb/e/e3/Hermitage_cat.jpeg/1200px-Hermitage_cat.jpeg",
  ],
}) => {
  let [index, setIndex] = useState(0);
  return (
    <div className="Gallery">
      <img src={images[index]} alt="myImage" />
      <Button
        variant="warning"
        onClick={
          index - 1 < 0
            ? () => setIndex(images.length - 1)
            : () => setIndex(index - 1)
        }
      >
        Предыдущая
      </Button>
      <Button
        variant="success"
        onClick={
          index + 1 === images.length
            ? () => setIndex(0)
            : () => setIndex(index + 1)
        }
      >
        Следующая
      </Button>
      <br />
      <Dots active={index} count={images.length} onClick={(i) => setIndex(i)} />
    </div>
  );
};

function App() {
  const [country, setCountry] = useState("UA");
  return (
    <div className="App">
      <h3>RGB Input:</h3>
      <RGBInput />

      <h3>Spoilers:</h3>
      <Spoiler header={<h1>Заголовок</h1>} open>
        <h2>Контент 1</h2>
        <p>
          Lorem ipsum dolor sit amet consectetur adipisicing elit. Laudantium
          eveniet pariatur ut debitis harum modi. Nesciunt magnam, reiciendis
          ipsa soluta reprehenderit iste necessitatibus fugiat non obcaecati
          officia delectus asperiores eaque numquam quae rem esse neque. Dolorum
          provident doloribus voluptate neque ea nostrum maxime enim. Magni
          debitis saepe nulla pariatur consequuntur.
        </p>
      </Spoiler>
      <Spoiler>
        <h2>Контент 2</h2>
        <p>
          Lorem ipsum, dolor sit amet consectetur adipisicing elit. Amet quo
          enim iusto quibusdam? Maxime amet asperiores numquam, eaque,
          reprehenderit hic provident, a praesentium iure aperiam sapiente?
          Earum aperiam sed odio facere, optio dignissimos debitis provident
          pariatur expedita iure obcaecati ut alias beatae, delectus
          accusantium, atque animi numquam. Aspernatur, placeat beatae!
        </p>
      </Spoiler>

      <h3>RangeInput:</h3>
      <RangeInput min={2} max={10} />

      <h3>PasswordConfirm:</h3>
      <PasswordConfirm min={4} />

      <h3>Timers:</h3>
      <Timers />

      <h3>TimerControl:</h3>
      <TimerControl />

      <h3>TimerContainer:</h3>
      <TimerContainer seconds={1000} refresh={100} render={SecondsTimer} />

      <h3>LCD:</h3>
      <TimerContainer seconds={3600} refresh={100} render={TimerLCD} />

      <h3>TimerControlContainer:</h3>
      <TimerControlContainer refresh={100} render={TimerLCD} />

      <h3>Watch:</h3>
      <TimerContainer seconds={10} refresh={100} render={Watch} mode="plus" />

      <h3>MySelect:</h3>
      <MySelect
        value={country}
        onChange={(newCountry) => setCountry(newCountry)}
      />

      <h3>Gallery:</h3>
      <Gallery />
    </div>
  );
}

export default App;
